/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalEnvelope.h"
#include "HvControlBinop.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvControlCast.h"
#include "HvControlVar.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_2ivJLEEY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_CDSiwEur_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_J9iwQixF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Qzw5hlvC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_csx9IwSB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oaugvFmf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MPBQlb5k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_7IoG2o5l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NOq8Fzhl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1gEf6BuR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_g6pO6EfR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_K2KJfkUm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_z3FjbZjt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_SlEcrpXe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_VtfBH7pI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_KvXHv14T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_o9RcdDam_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_qN78YpwT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_HKhzLtJ6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nLdEYOmM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_lrNaITuR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_haaFyG5z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qqowG0RV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_68ZpTQuX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_UzBNDZOW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3VO8qZCw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_h7yyCHmD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8vIUCwog_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_FvSTJRfc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_TiIj0ag2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_sjBPudHO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_rx9klRQg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_bWnzDXLN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_vEuOo3Ii_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_kym7nGfu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_mG81oLRj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4e7sZBDf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_xuxs6RqB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kQNKB00Q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_gDFjb7wp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Cmxsu9Ow_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_NA1N0mC3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_yZZEgXjV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gjoPvtE0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_sZ1xO7Ya_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QOuvV3oU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_BrYynmfo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_xM3qjSG4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_qSrKAmnm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_w6cUdotS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_grPrBRPg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CSZBB5ly_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_WyFLnjnR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_af3FKjLR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_JBy16IA3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_rpqW0dZy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_XqaWlnnm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_BkE8ZUn7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ar5QncNP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_WstXccxj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_HZTLp6a0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_R0AaiPHW_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_hKm7BdSu;
  SignalEnvelope sEnv_qN78YpwT;
  SignalEnvelope sEnv_HKhzLtJ6;
  SignalEnvelope sEnv_kym7nGfu;
  SignalEnvelope sEnv_mG81oLRj;
  SignalLine sLine_hG66Zfwd;
  SignalLine sLine_gzVZgb1b;
  ControlBinop cBinop_2ivJLEEY;
  ControlBinop cBinop_CDSiwEur;
  ControlBinop cBinop_J9iwQixF;
  ControlBinop cBinop_Qzw5hlvC;
  ControlBinop cBinop_MPBQlb5k;
  ControlBinop cBinop_NOq8Fzhl;
  ControlBinop cBinop_1gEf6BuR;
  ControlBinop cBinop_g6pO6EfR;
  ControlVar cVar_K2KJfkUm;
  ControlVar cVar_SlEcrpXe;
  SignalVarf sVarf_7Jv24se5;
  ControlVar cVar_VtfBH7pI;
  ControlIf cIf_KvXHv14T;
  ControlIf cIf_o9RcdDam;
  ControlBinop cBinop_nLdEYOmM;
  ControlBinop cBinop_haaFyG5z;
  ControlBinop cBinop_qqowG0RV;
  ControlBinop cBinop_3VO8qZCw;
  ControlBinop cBinop_8vIUCwog;
  SignalVarf sVarf_FYlPRP1K;
  ControlVar cVar_rx9klRQg;
  ControlIf cIf_bWnzDXLN;
  ControlIf cIf_vEuOo3Ii;
  ControlBinop cBinop_4e7sZBDf;
  ControlBinop cBinop_kQNKB00Q;
  ControlBinop cBinop_gDFjb7wp;
  ControlBinop cBinop_yZZEgXjV;
  ControlBinop cBinop_sZ1xO7Ya;
  ControlVar cVar_qSrKAmnm;
  ControlBinop cBinop_w6cUdotS;
  SignalVarf sVarf_Ol5qfsPI;
  SignalVarf sVarf_LPK2PMFC;
  SignalVarf sVarf_QtUFR1xV;
  SignalVarf sVarf_Ld9AQncB;
  ControlBinop cBinop_af3FKjLR;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
